package aLibrary;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.Serializable;

/*	AButton objects are an ALibrary version of the swing JButton 
	Author:  David Riley -- July, 2000   

	class invariant
		getWidth() >= 0  AND  getHeight() >= 0  */
public class AButton extends JButton  
	implements AShape, ActionListener, Serializable  {
	
	// The following variables implement a unique ID number for each AButton 
	private static int idShared = 0;
	private int idNum;

// Attributes   
	/* 	postcondition
			this is placed  IMPLIES  
				result == the parent object upon which this is placed
			AND  this not placed  IMPLIES  result == null   */
	public Container parentContainer()  {
		Container c;
		c = getParent();
		if (c instanceof APanel)   {
			return ((APanel)c).getAWindow();
		} else {
			return c;
		}
	} 


// Constructor methods
	/* 	postcondition
			getX() == 0  AND  getY() == 0  
			AND getWidth() == 10  AND  getHeight() == 10  
			AND getText() == ""    */
	public AButton()  {
        super();
		Font newFont = new Font( getFont().getName(), Font.PLAIN, 10 );
		setFont( newFont );
		setText("");
		addActionListener(this);
		setBounds(0, 0, 10, 10);
		idShared++;
		idNum = idShared;
	}  

	/*	precondition
			w >= 0  AND  h >= 0  
		postcondition
			getX() == x  AND  getY() == y  
			AND getWidth() == w  AND  getHeight() == h  
			AND getText() == ""    */
	public AButton(int x, int y, int w, int h)  {
        super();
		Font newFont = new Font( getFont().getName(), Font.PLAIN, 10 );
		setFont( newFont );
		setText("");
		addActionListener(this);
		setBounds(x, y, w, h);
		idShared++;
		idNum = idShared;
	} 

	/* 	precondition
			w >= 0  AND  h >= 0  
		postcondition
			getX() == x  AND  getY() == y  
			AND getWidth() == w  AND  getHeight() == h  
			AND getText() == s    */
	public AButton(String s, int x, int y, int w, int h)  {
        super();
		Font newFont = new Font( getFont().getName(), Font.PLAIN, 10 );
		setFont( newFont );
		setText(s);
		addActionListener(this);
		setBounds(x, y, w, h);
		idShared++;
		idNum = idShared;
	}


// Class Methods
	/* 	postcondition
			this is added to the c container */
	public void place(Container c)  {
		c.add(this,0);  //zero places at front of list
	}

	/* 	postcondition
			this is removed from where it is placed  
			AND  parentContainer() == null  */
	public void remove()  {
		if  (getParent() != null)
			getParent().remove(this);
	} 

	/* 	precondition
			sw >= 0.0  AND  sh >= 0.0   
		postcondition
			getWidth() == old getWidth() * sw   
			AND   getHeight() == old getHeight() * sh */
	public void scale(double sw, double sh)  {
		setBounds(getX(), getY(),
				(int)(getWidth() * sw),
				(int)(getHeight() * sh));
	} 

	/* 	postcondition
			getX() == old getX() + tx   
			AND   getY() == old getY() + ty  */
	public void translate(double tx, double ty)  {
		setLocation((int)(getX() + tx), (int)(getY() + ty));
	} 
   
	/* 	postcondition
			result == the attributes of this object   */
	public String toString()   {
		String  result = "AButton object .... (#B" + idNum + ")" + "\n";
   		result = result + "     label: " + getText() +"\n";
   		result = result + "     x: " + getX() +",   y: "+getY();
   		result = result + "     width: " + getWidth() +",   height: "+getHeight() + "\n";
		if (parentContainer() == null)   {
			result = result + "     not placed, ";
   		} else   {
			result = result + "     is placed, ";
   		}
		return result;
   	}	  	

   		

/* Action event handler (to be redefined by inheriting classes) */
	public void actionPerformed(ActionEvent e)
	{} 
   
}