package aLibrary;

/*	AImage objects are invisible panes for displaying gif or jpeg images 
    Author:  David Riley -- July, 2000 */
    
/*	class invariant
    	getWidth() >= 0  AND  getHeight() >= 0  */

import javax.swing.event.*;
import java.awt.event.*;
import java.awt.Toolkit.*;
import java.awt.*;
import java.io.Serializable;

public class AImage extends AView  
    				implements AShape, MouseListener, MouseMotionListener, Serializable  {
	private Image content;
	// The variables implement a unique ID number for each AImage 
	private static int idShared = 0;
	private int idNum;

// Constructor methods
	/*	postcondition 
			an empty image view is instantiated
			AND  getX() == 0   AND  getY() == 0   
			AND  getWidth() == 10  AND  getHeight() == 10   */
	public AImage()  {
		super();
		AImage.idShared++;
		idNum = AImage.idShared;
		setBounds(0, 0, 10, 10);
	} 

	/*	precondition
			w >= 0   AND   h >= 0
		postcondition 
			an empty image view is instantiated
			AND  getX() == x   AND  getY() == y   
			AND  getWidth() == w  AND  getHeight() == h   */
	public AImage(int x, int y, int w, int h)  {
		super();
		AImage.idShared++;
		idNum = AImage.idShared;
		setBounds(x, y, w, h);
	} 



// Class Methods   

	/*	postcondition
			Upon repaint the image displayed will be given by the 
			file with complete pathname specified as s 
			(Note that the file should be a gif or jpeg encoding.)  */
	public void setImage(String s)  {
        java.net.URL url = getClass().getResource(s);
        if (url == null)   {
            url = getClass().getResource("/"+s);
        }
        content = getToolkit().getImage(url);
	}
   
	/* 	postcondition
			result == the attributes of this object   */
	public String toString()   {
    	String  result = "AImage object .... (#I" + idNum + ")" + "\n";
   		result = result + "     x: " + getX() +",   y: "+getY();
   		result = result + "     width: " + getWidth() +",   height: "+getHeight() + "\n";
		if (parentContainer() == null)   {
   			result = result + "     not placed, ";
   		} else   {
   			result = result + "     is placed, ";
   		}
		return result;
   	}	


//Auxiliary methods
	public void fill(Graphics g)  {
		g.drawImage(content, 0, 0, getWidth(), getHeight(), this);
	} 

	public void stroke(Graphics g)  {
		g.drawImage(content, 0,0, getWidth(), getHeight(), this);
	}



// Mouse event handlers
	public void mouseClicked(MouseEvent e)
	{}

	public void mousePressed(MouseEvent e)
	{}

	public void mouseReleased(MouseEvent e)
	{}

	public void mouseEntered(MouseEvent e)
	{}

	public void mouseExited(MouseEvent e)
	{}

// Mouse motion event handlers
	public void mouseDragged(MouseEvent e)
	{}

	public void mouseMoved(MouseEvent e)
	{}
}