package aLibrary;

import java.awt.*;
import javax.swing.*;
import java.lang.*;
import java.io.Serializable;

/*	ALabel objects are for displaying strings of text 
    Author:  David Riley -- July, 2000 */
    
/*	class invariant
		getWidth() >= 0  AND  getHeight() >= 0  */
public class ALabel extends JLabel implements AShape,  Serializable  {
	// The variables implement a unique ID number for each ARectangle 
	private static int idShared = 0;
	private int idNum;

// Attribute
	/* 	postcondition
   			result == the parent object upon which this is placed   */
	public Container parentContainer()  {
		Container c;
   		c = getParent();
   		if (c instanceof APanel)   {
			return ((APanel)c).getAWindow();
   		} else {
			return c;
		}
	} 


// Constructor methods
	/*	postcondition 
   			this ALabel is instantiated
			AND  getColor() == Color.black 
			AND  getX() == 0   AND  getY() == 0   
			AND  getWidth() == 100  AND  getHeight() == 15
			AND  getFontSize() == 10   */
	public ALabel()  {
		super("");
		setBounds(0, 0, 100, 15);
		Font newFont = new Font( "Courier", Font.PLAIN, 10 );
		setFont( newFont );
		setAlignmentLeft();
		setColor(Color.black);
		idShared++;
		idNum = idShared;
	} 

	/*	postcondition 
			this ALabel is instantiated
			AND  getColor() == Color.black 
			AND  getX() == x   AND  getY() == y   
			AND  getWidth() == w  AND  getHeight() == h  
			AND  getFontSize() == 10   */
	public ALabel(int x, int y, int w, int h)  {
		super("");
		setBounds(x, y, w, h);
		Font newFont = new Font( "Courier", Font.PLAIN, 10 );
		setFont( newFont );
		setAlignmentLeft();
		setColor(Color.black);
		idShared++;
		idNum = idShared;
	} 


// Class Methods
	/* 	postcondition
			this is added to the c container */
	public void place(Container c)  {
		c.add(this,0);  //zero places at front of list
		// It's not clear why, but the following three instructions
		// appear to compensate for an updating problem for Labels
		// in Version 1.2
		String tmp = getText();
		setText("xxx");
		setText(tmp);
	}

	/*	postcondition
			this is removed from where it is placed  
			AND  parentContainer() == null  */
	public void remove()  {
		if  (getParent() != null)
			getParent().remove(this);
	} 

	/* 	precondition
			sw >= 0.0  AND  sh >= 0.0   
		postcondition
			getWidth() == old getWidth() * sw   
			AND   getHeight() == old getHeight() * sh */
	public void scale(double sw, double sh)  {
		setBounds(getX(), getY(),
					(int)(getWidth() * sw),
					(int)(getHeight() * sh));
	} 

	/* 	postcondition
			getX() == old getX() + tx   
			AND   getY() == old getY() + ty  */
	public void translate(double tx, double ty)  {
		setLocation((int)(getX() + tx), (int)(getY() + ty));
	} 

	/* 	postcondition
			result == the attributes of this object   */
	public String toString()   {
		String  result = "ALabel object .... (#LA" + idNum + ")" + "\n";
   		result = result + "     text: " + "\""+getText() +"\"" + "\n";
   		result = result + "     x: " + getX() +",   y: "+getY();
   		result = result + "     width: " + getWidth() +",   height: "+getHeight() + "\n";
		if (parentContainer() == null)   {
   			result = result + "     not placed, " + "\n";
   		} else   {
   			result = result + "     is placed, " + "\n";
   		}
   		result = result + "     color: "+colorString(getColor());
		return result;
   	}	
   	
	/* 	postcondition
   			result == the color attribute of this */
	public Color getColor()  {
		return getForeground();
	} 

	/*	postcondition
			getColor() == c */
	public void setColor(Color c)  {
		setForeground(c);
	}

        /*	postcondition
   			The size of the characters is set to a font size of s */
	public void setFontSize( int s )  {
		Font newFont = new Font( getFont().getName(), Font.PLAIN, s );
		setFont( newFont );
	} 
   
	/*	postcondition
   			result == font size for this */
	public int getFontSize( int s )  {
		return getFont().getSize();
	} 
   
	/*	postcondition
   			textValue() is displayed in left-justified format */
	public void setAlignmentLeft()  {
		setHorizontalAlignment(LEFT);
	} 

	/*	postcondition
   			textValue() is displayed in right-justified format */
	public void setAlignmentRight()  {
		setHorizontalAlignment(RIGHT);
	} 

	/*	postcondition
   			textValue() is displayed in centered format */
	public void setAlignmentCenter()  {
		setHorizontalAlignment(CENTER);
	} 


// private methods
	private String colorString( Color c )   {
		if (c == Color.black)   {
			return "black";
		} else if (c == Color.blue)   {
			return "blue";
		} else if (c == Color.cyan)   {
			return "cyan";
		} else if (c == Color.darkGray)   {
			return "dark gray";
		} else if (c == Color.gray)   {
			return "gray";
		} else if (c == Color.green)   {
			return "green";
		} else if (c == Color.lightGray)   {
			return "light gray";
		} else if (c == Color.magenta)   {
			return "magenta";
		} else if (c == Color.orange)   {
			return "orange";
		} else if (c == Color.pink)   {
			return "pink";
		} else if (c == Color.red)   {
			return "red";
		} else if (c == Color.white)   {
			return "white";
		} else if (c == Color.yellow)   {
			return "yellow";
		} else {
			return "redness: "+c.getRed() + "   greeness: "+c.getGreen()+"   blueness: "+c.getBlue();
		}
	}

}
