package aLibrary;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.Serializable;

/*	AScrollbar objects are an ALibrary version of the swing JScrollBar 
    Author:  David Riley -- July, 2000 */
    
/*	class invariant
		getWidth() >= 0  AND  getHeight() >= 0
    	AND  getMinimum() <= getValue() <= getMaximum()  
    	AND  getWidth() >= getHeight()  implies  horizontal dragging
    	AND  getWidth() < getHeight()  implies  vertical dragging */
public class AScrollbar extends JScrollBar  
	 implements AShape,  Serializable, AdjustmentListener  {
   
	// The variables implement a unique ID number for each ARectangle 
	private static int idShared = 0;
	private int idNum;

// Attribute
	/* 	postcondition
   			result == the parent object upon which this is placed   */
	public Container parentContainer()  {
		Container c;
   		c = getParent();
   		if (c instanceof APanel)   {
   			return ((APanel)c).getAWindow();
   		} else {
   			return c;
   		}
	} 


// Constructor methods
	/*	postcondition 
			getX() == 0   AND  getY() == 0   
			AND  getWidth() == 100  AND  getHeight() == 20   
			AND  getMinimum() == 1  AND  getMaximum() == 100
			AND  getValue() == 1 */
	public AScrollbar()  {
		super(HORIZONTAL, 1, 0, 1, 100);
		setBounds(0, 0, 100, 20);
		addAdjustmentListener(this);
		idShared++;
		idNum = idShared;
	} 

	/*	postcondition 
			getX() == x   AND  getY() == y   
			AND  getWidth() == w  AND  getHeight() == h   
			AND  getMinimum() == 1  AND  getMaximum() == 100
			AND  getValue() == 1 */
	public AScrollbar(int x, int y, int w, int h)  {
		super(HORIZONTAL, 1, 0, 1, 100);
		setBounds(x, y, w, h);
		addAdjustmentListener(this);
		idShared++;
		idNum = idShared;
	} 


// Class Methods
	/*	precondition
   			mn <= v <= mx
   	   	postcondition
   	   		getMinimum() = mn  AND  getMaximum() == mx
   	   		AND getValue() == v      */
	public void setMinMaxVal(int mn, int mx, int v)  {
		setValues(v, 0, mn, mx);
	}

	/* 	precondition
   			sw >= 0.0  AND  sh >= 0.0   
     	 postcondition
      		getWidth() == old getWidth() * sw   
      		AND   getHeight() == old getHeight() * sh */
	public void scale(double sw, double sh)  {
		setBounds(getX(), getY(),
            	(int)(getWidth() * sw),
             	(int)(getHeight() * sh));
	} 

	/* 	postcondition
   			getX() == old getX() + tx   
   			AND   getY() == old getY() + ty  */
	public void translate(double tx, double ty)  {
		setLocation((int)(getX() + tx), (int)(getY() + ty));
	} 
   
	/* 	postcondition
   			this is added to the c container */
	public void place(Container c)  {
		c.add(this,0);  //zero places at front of list
	}

	/* postcondition
			this is removed from where it is placed  
			AND  parentContainer() == null  */
	public void remove()  {
		if  (getParent() != null)
			getParent().remove(this);
	}

	/* 	postcondition
   			getWidth() == w   AND   getHeight() == h  */
	public void setSize(int w, int h)  {
		setBounds(getX(), getY(), w, h);
	}

	/* 	postcondition
   			getX() == y  AND  getY() == y
   			AND getWidth() == w   AND   getHeight() == h
   			AND  (w >= h) IMPLIES horizontal orientation
   			AND  (w < h) IMPLIES vertical orientation  */
	public void setBounds(int x, int y, int w, int h)  {
		super.setBounds(x, y, w, h);
		if (w >= h)
			setOrientation(HORIZONTAL);
		else
			setOrientation(VERTICAL);
	}

	/* 	postcondition
			result == the attributes of this object   */
	public String toString()   {
		String  result = "AScrollbar object .... (#S" + idNum + ")" +"\n";
   		result = result + "     x: " + getX() +",   y: "+getY();
   		result = result + "     width: " + getWidth() +",   height: "+getHeight() +"\n";
   		result = result + "     value: " + getValue()+",";
   		result = result + "   min: " + getMinimum() +",   max: "+getMaximum() +"\n";
		if (parentContainer() == null)   {
   			result = result + "     not placed, ";
   		} else   {
   			result = result + "     is placed, ";
   		}
		return result;
   	}	


// Action event handler
	public void adjustmentValueChanged(AdjustmentEvent e)
	{} 
}