package aLibrary;

import javax.swing.*;
import java.awt.*;
import java.io.Serializable;

/*	ATextArea objects are designed to display lines of output text 
	in a scrollable object.  ATextAreas do NOT support input.
    Author:  David Riley -- July, 2000 */
    
/*	class invariant
		getWidth() >= 0  AND  getHeight() >= 0  */
public class ATextArea extends TextArea  
   			 implements AShape,  Serializable  {
	// The variables implement a unique ID number for each ARectangle 
	private static int idShared = 0;
	private int idNum;

// Attribute
	/* 	postcondition
			result == the parent object upon which this is placed   */
	public Container parentContainer()  {
		Container c;
   		c = getParent();
   		if (c instanceof APanel)   {
   			return ((APanel)c).getAWindow();
   		} else {
   			return c;
   		}
	} 


// Constructor methods
	/*	postcondition 
   			this ATextArea object is instantiated
			AND  getForeground() == Color.black 
			AND  getBackground() == Color.white
			AND  getX() == 0   AND  getY() == 0   
			AND  getWidth() == 100  AND  getHeight() == 100   */
	public ATextArea()  {
		super();
		setBounds(0, 0, 100, 100);
		setVisible(true);
		setForeground(Color.black);
		setBackground(Color.white);
		Font newFont = new Font( "Courier", Font.PLAIN, 10 );
		setFont( newFont );
		setEditable(false);
		idShared++;
		idNum = idShared;
	} 

	/*	postcondition 
   			this ATextArea object is instantiated
			AND  getForeground() == Color.black 
			AND  getBackground() == Color.white
			AND  getX() == x   AND  getY() == y   
			AND  getWidth() == w  AND  getHeight() == h   */
	public ATextArea(int x, int y, int w, int h)  {
		super();
		setBounds(x, y, w, h);
		setVisible(true);
		setForeground(Color.black);
		setBackground(Color.lightGray);
		Font newFont = new Font( "Courier", Font.PLAIN, 10 );
		setFont( newFont );
		setEditable(false);
		idShared++;
		idNum = idShared;
	} 


// Class Methods:
	/* 	precondition
   			sw >= 0.0  AND  sh >= 0.0   
     	 postcondition
      		getWidth() == old getWidth() * sw   
      		AND   getHeight() == old getHeight() * sh */
	public void scale(double sw, double sh)  {
		setBounds(getX(), getY(),
            	(int)(getWidth() * sw),
             	(int)(getHeight() * sh));
	} 

	/* 	postcondition
   			getX() == old getX() + tx   
   			AND   getY() == old getY() + ty  */
	public void translate(double tx, double ty)  {
		setLocation((int)(getX() + tx), (int)(getY() + ty));
	} 
   
	/* 	postcondition
   			this is added to the c container */
	public void place(Container c)  {
		c.add(this,0);  //zero places at front of list
	}

	/* postcondition
   		this is removed from where it is placed  
		AND  parentContainer() == null  */
	public void remove()  {
		if  (getParent() != null)
			getParent().remove(this);
	}

	/*	postcondition 
   			getText() == old getText() + s + "\n"  */
	public void appendln(String s)  {
		append(s + "\n");
	}  
   
	/* 	postcondition
			result == the attributes of this object   */
	public String toString()   {
		String  result = "ATextArea object .... (#TA" + idNum + ")" + "\n";
   		result = result + "     x: " + getX() +",   y: "+getY();
   		result = result + "     width: " + getWidth() +",   height: "+getHeight() + "\n";
		if (parentContainer() == null)   {
   			result = result + "     not placed" + "\n";
   		} else   {
   			result = result + "     is placed" + "\n";
   		}
   		result = result + "     foreground color: "+colorString(getForeground());
   		result = result + ",   background color: "+colorString(getBackground());
		return result;
   	}	
   	
	/*	postcondition
   			The size of the characters is set to a font size of s */
	public void setFontSize( int s )  {
		Font newFont = new Font( getFont().getName(), Font.PLAIN, s );
		setFont( newFont );
	} 
   
    /*	postcondition
   			result == font size for this */
	public int getFontSize( int s )  {
		return getFont().getSize();
	} 
   	
// private methods
	private String colorString( Color c )   {
		if (c == Color.black)   {
			return "black";
		} else if (c == Color.blue)   {
			return "blue";
		} else if (c == Color.cyan)   {
			return "cyan";
		} else if (c == Color.darkGray)   {
			return "dark gray";
		} else if (c == Color.gray)   {
			return "gray";
		} else if (c == Color.green)   {
			return "green";
		} else if (c == Color.lightGray)   {
			return "light gray";
		} else if (c == Color.magenta)   {
			return "magenta";
		} else if (c == Color.orange)   {
			return "orange";
		} else if (c == Color.pink)   {
			return "pink";
		} else if (c == Color.red)   {
			return "red";
		} else if (c == Color.white)   {
			return "white";
		} else if (c == Color.yellow)   {
			return "yellow";
		} else {
			return "redness: "+c.getRed() + "   greeness: "+c.getGreen()+"   blueness: "+c.getBlue();
		}
	}
}